{% if config.predicate_table %}
    {% include 'predicate_table.html.jinja' %}
{% endif %}

{% if config.dependency_graph %}
    {% include 'dependency_graph.html.jinja' %}
{% endif %}

{% if config.encodings %}
    {% include 'encodings.html.jinja' %}
{% endif %}

{% if config.glossary %}
    {% include 'separator.html.jinja' %}

    {% include 'glossary.html.jinja' %}
{% endif %}
